@@ -28,7 +28,8 @@
 #include "nm-setting-connection.h"
 #include "nm-setting-olpc-mesh.h"
 #include "nm-manager.h"
-#include "platform/nm-platform.h"
+#include "libnm-core-aux-intern/nm-libnm-core-utils.h"
+#include "libnm-platform/nm-platform.h"
 
 #define _NMLOG_DEVICE_TYPE NMDeviceOlpcMesh
 #include "devices/nm-device-logging.h"
@@ -38,7 +39,7 @@
 NM_GOBJECT_PROPERTIES_DEFINE(NMDeviceOlpcMesh, PROP_COMPANION, PROP_ACTIVE_CHANNEL, );
 
 typedef struct {
-    NMDevice * companion;
+    NMDevice  *companion;
     NMManager *manager;
     bool       stage1_waiting : 1;
 } NMDeviceOlpcMeshPrivate;
@@ -62,7 +63,7 @@ G_DEFINE_TYPE(NMDeviceOlpcMesh, nm_device_olpc_mesh, NM_TYPE_DEVICE)
 static gboolean
 get_autoconnect_allowed(NMDevice *device)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(device);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(device);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     /* We can't even connect if we don't have a companion yet. */
@@ -80,19 +81,15 @@ get_autoconnect_allowed(NMDevice *device)
 #define DEFAULT_SSID "olpc-mesh"
 
 static gboolean
-complete_connection(NMDevice *           device,
-                    NMConnection *       connection,
-                    const char *         specific_object,
+complete_connection(NMDevice            *device,
+                    NMConnection        *connection,
+                    const char          *specific_object,
                     NMConnection *const *existing_connections,
-                    GError **            error)
+                    GError             **error)
 {
     NMSettingOlpcMesh *s_mesh;
 
-    s_mesh = nm_connection_get_setting_olpc_mesh(connection);
-    if (!s_mesh) {
-        s_mesh = (NMSettingOlpcMesh *) nm_setting_olpc_mesh_new();
-        nm_connection_add_setting(connection, NM_SETTING(s_mesh));
-    }
+    s_mesh = _nm_connection_ensure_setting(connection, NM_TYPE_SETTING_OLPC_MESH);
 
     if (!nm_setting_olpc_mesh_get_ssid(s_mesh)) {
         gs_unref_bytes GBytes *ssid = NULL;
@@ -122,10 +119,21 @@ complete_connection(NMDevice *           device,
 
 /*****************************************************************************/
 
+static const char *
+get_dhcp_anycast_address(NMDevice *device)
+{
+    NMSettingOlpcMesh *s_mesh;
+
+    s_mesh = nm_device_get_applied_setting(device, NM_TYPE_SETTING_OLPC_MESH);
+    return s_mesh ? nm_setting_olpc_mesh_get_dhcp_anycast_address(s_mesh) : NULL;
+}
+
+/*****************************************************************************/
+
 static NMActStageReturn
 act_stage1_prepare(NMDevice *device, NMDeviceStateReason *out_failure_reason)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(device);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(device);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     /* disconnect companion device, if it is connected */
@@ -175,10 +183,9 @@ _mesh_set_channel(NMDeviceOlpcMesh *self, guint32 channel)
 static NMActStageReturn
 act_stage2_config(NMDevice *device, NMDeviceStateReason *out_failure_reason)
 {
-    NMDeviceOlpcMesh * self = NM_DEVICE_OLPC_MESH(device);
+    NMDeviceOlpcMesh  *self = NM_DEVICE_OLPC_MESH(device);
     NMSettingOlpcMesh *s_mesh;
-    GBytes *           ssid;
-    const char *       anycast_addr;
+    GBytes            *ssid;
     gboolean           success;
 
     s_mesh = nm_device_get_applied_setting(device, NM_TYPE_SETTING_OLPC_MESH);
@@ -191,15 +198,12 @@ act_stage2_config(NMDevice *device, NMDeviceStateReason *out_failure_reason)
                                         nm_device_get_ifindex(device),
                                         g_bytes_get_data(ssid, NULL),
                                         g_bytes_get_size(ssid));
-    nm_device_bring_up(NM_DEVICE(self), TRUE, NULL);
+    nm_device_bring_up(NM_DEVICE(self));
     if (!success) {
         _LOGW(LOGD_WIFI, "Unable to set the mesh ID");
         return NM_ACT_STAGE_RETURN_FAILURE;
     }
 
-    anycast_addr = nm_setting_olpc_mesh_get_dhcp_anycast_address(s_mesh);
-    nm_device_set_dhcp_anycast_address(device, anycast_addr);
-
     if (!_mesh_set_channel(self, nm_setting_olpc_mesh_get_channel(s_mesh))) {
         _LOGW(LOGD_WIFI, "Unable to set the mesh channel");
         return NM_ACT_STAGE_RETURN_FAILURE;
@@ -239,7 +243,7 @@ companion_cleanup(NMDeviceOlpcMesh *self)
 static void
 companion_notify_cb(NMDeviceWifi *companion, GParamSpec *pspec, gpointer user_data)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(user_data);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(user_data);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     nm_assert(NM_IS_DEVICE_WIFI(companion));
@@ -256,7 +260,7 @@ companion_notify_cb(NMDeviceWifi *companion, GParamSpec *pspec, gpointer user_da
 
 /* disconnect from mesh if someone starts using the companion */
 static void
-companion_state_changed_cb(NMDeviceWifi *      companion,
+companion_state_changed_cb(NMDeviceWifi       *companion,
                            NMDeviceState       state,
                            NMDeviceState       old_state,
                            NMDeviceStateReason reason,
@@ -266,7 +270,7 @@ companion_state_changed_cb(NMDeviceWifi *      companion,
     NMDeviceState     self_state = nm_device_get_state(NM_DEVICE(self));
 
     if (old_state > NM_DEVICE_STATE_DISCONNECTED && state <= NM_DEVICE_STATE_DISCONNECTED) {
-        nm_device_emit_recheck_auto_activate(NM_DEVICE(self));
+        nm_device_recheck_auto_activate_schedule(NM_DEVICE(self));
     }
 
     if (self_state < NM_DEVICE_STATE_PREPARE || self_state > NM_DEVICE_STATE_ACTIVATED
@@ -295,7 +299,7 @@ static gboolean
 check_companion(NMDeviceOlpcMesh *self, NMDevice *other)
 {
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
-    const char *             my_addr, *their_addr;
+    const char              *my_addr, *their_addr;
 
     if (!NM_IS_DEVICE_WIFI(other))
         return FALSE;
@@ -333,7 +337,7 @@ check_companion(NMDeviceOlpcMesh *self, NMDevice *other)
 static void
 device_added_cb(NMManager *manager, NMDevice *other, gpointer user_data)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(user_data);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(user_data);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     if (!priv->companion && check_companion(self, other)) {
@@ -359,8 +363,8 @@ static void
 find_companion(NMDeviceOlpcMesh *self)
 {
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
-    const CList *            tmp_lst;
-    NMDevice *               candidate;
+    const CList             *tmp_lst;
+    NMDevice                *candidate;
 
     if (priv->companion)
         return;
@@ -382,12 +386,12 @@ find_companion(NMDeviceOlpcMesh *self)
 }
 
 static void
-state_changed(NMDevice *          device,
+state_changed(NMDevice           *device,
               NMDeviceState       new_state,
               NMDeviceState       old_state,
               NMDeviceStateReason reason)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(device);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(device);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     if (new_state == NM_DEVICE_STATE_UNAVAILABLE)
@@ -418,8 +422,8 @@ get_dhcp_timeout_for_device(NMDevice *device, int addr_family)
 static void
 get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-    NMDeviceOlpcMesh *       self   = NM_DEVICE_OLPC_MESH(object);
-    NMDevice *               device = NM_DEVICE(self);
+    NMDeviceOlpcMesh        *self   = NM_DEVICE_OLPC_MESH(object);
+    NMDevice                *device = NM_DEVICE(self);
     NMDeviceOlpcMeshPrivate *priv   = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     switch (prop_id) {
@@ -446,7 +450,7 @@ nm_device_olpc_mesh_init(NMDeviceOlpcMesh *self)
 static void
 constructed(GObject *object)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(object);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(object);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     G_OBJECT_CLASS(nm_device_olpc_mesh_parent_class)->constructed(object);
@@ -475,7 +479,7 @@ nm_device_olpc_mesh_new(const char *iface)
 static void
 dispose(GObject *object)
 {
-    NMDeviceOlpcMesh *       self = NM_DEVICE_OLPC_MESH(object);
+    NMDeviceOlpcMesh        *self = NM_DEVICE_OLPC_MESH(object);
     NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE(self);
 
     companion_cleanup(self);
@@ -492,27 +496,27 @@ dispose(GObject *object)
 static const NMDBusInterfaceInfoExtended interface_info_device_olpc_mesh = {
     .parent = NM_DEFINE_GDBUS_INTERFACE_INFO_INIT(
         NM_DBUS_INTERFACE_DEVICE_OLPC_MESH,
-        .signals    = NM_DEFINE_GDBUS_SIGNAL_INFOS(&nm_signal_info_property_changed_legacy, ),
         .properties = NM_DEFINE_GDBUS_PROPERTY_INFOS(
-            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE_L("HwAddress",
-                                                             "s",
-                                                             NM_DEVICE_HW_ADDRESS),
-            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE_L("Companion",
-                                                             "o",
-                                                             NM_DEVICE_OLPC_MESH_COMPANION),
-            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE_L(
+            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE(
+                "HwAddress",
+                "s",
+                NM_DEVICE_HW_ADDRESS,
+                .annotations = NM_GDBUS_ANNOTATION_INFO_LIST_DEPRECATED(), ),
+            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE("Companion",
+                                                           "o",
+                                                           NM_DEVICE_OLPC_MESH_COMPANION),
+            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE(
                 "ActiveChannel",
                 "u",
                 NM_DEVICE_OLPC_MESH_ACTIVE_CHANNEL), ), ),
-    .legacy_property_changed = TRUE,
 };
 
 static void
 nm_device_olpc_mesh_class_init(NMDeviceOlpcMeshClass *klass)
 {
-    GObjectClass *     object_class      = G_OBJECT_CLASS(klass);
+    GObjectClass      *object_class      = G_OBJECT_CLASS(klass);
     NMDBusObjectClass *dbus_object_class = NM_DBUS_OBJECT_CLASS(klass);
-    NMDeviceClass *    device_class      = NM_DEVICE_CLASS(klass);
+    NMDeviceClass     *device_class      = NM_DEVICE_CLASS(klass);
 
     object_class->constructed  = constructed;
     object_class->get_property = get_property;
@@ -531,6 +535,7 @@ nm_device_olpc_mesh_class_init(NMDeviceOlpcMeshClass *klass)
     device_class->act_stage2_config           = act_stage2_config;
     device_class->state_changed               = state_changed;
     device_class->get_dhcp_timeout_for_device = get_dhcp_timeout_for_device;
+    device_class->get_dhcp_anycast_address    = get_dhcp_anycast_address;
 
     obj_properties[PROP_COMPANION] = g_param_spec_string(NM_DEVICE_OLPC_MESH_COMPANION,
                                                          "",
