@@ -161,7 +161,7 @@ receive_ra(struct ndp *ndp, struct ndp_msg *msg, gpointer user_data)
          * let's keep the "most managed" level. */
         G_STATIC_ASSERT_EXPR(NM_NDISC_DHCP_LEVEL_MANAGED > NM_NDISC_DHCP_LEVEL_OTHERCONF);
         G_STATIC_ASSERT_EXPR(NM_NDISC_DHCP_LEVEL_OTHERCONF > NM_NDISC_DHCP_LEVEL_NONE);
-        dhcp_level = MAX(dhcp_level, rdata->public.dhcp_level);
+        dhcp_level = NM_MAX(dhcp_level, rdata->public.dhcp_level);
 
         if (dhcp_level != rdata->public.dhcp_level) {
             rdata->public.dhcp_level = dhcp_level;
