@@ -10,9 +10,10 @@
 #define NM_L3CFG_CONFIG_PRIORITY_IPV6LL 1
 #define NM_L3CFG_CONFIG_PRIORITY_VPN    9
 #define NM_ACD_TIMEOUT_RFC5227_MSEC     9000u
+#define NM_ACD_TIMEOUT_MAX_MSEC         30000u
 
 #define NM_TYPE_L3CFG            (nm_l3cfg_get_type())
-#define NM_L3CFG(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_L3CFG, NML3Cfg))
+#define NM_L3CFG(obj)            (_NM_G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_L3CFG, NML3Cfg))
 #define NM_L3CFG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), NM_TYPE_L3CFG, NML3CfgClass))
 #define NM_IS_L3CFG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), NM_TYPE_L3CFG))
 #define NM_IS_L3CFG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), NM_TYPE_L3CFG))
@@ -33,8 +34,8 @@ typedef enum _nm_packed {
 /**
  * NML3CfgConfigFlags:
  * @NM_L3CFG_CONFIG_FLAGS_NONE: no flags, the default.
- * @NM_L3_CONFIG_MERGE_FLAGS_ONLY_FOR_ACD: if this merge flag is set,
- *   the the NML3ConfigData doesn't get merged and it's information won't be
+ * @NM_L3CFG_CONFIG_FLAGS_ONLY_FOR_ACD: if this flag is set,
+ *   the NML3ConfigData doesn't get merged and the information won't be
  *   synced. The only purpose is to run ACD on its IPv4 addresses, but
  *   regardless whether ACD succeeds/fails, the IP addresses won't be configured.
  *   The point is to run ACD first (without configuring it), and only
@@ -54,15 +55,11 @@ typedef enum _nm_packed {
  *   "don't change" behavior. At least once. If the address/route
  *   is still not (no longer) configured on the subsequent
  *   commit, it's not getting added again.
- * @NM_L3CFG_CONFIG_FLAGS_FORCE_ONCE: if set, objects in the
- *   NML3ConfigData are committed to platform even if they were
- *   removed externally.
  */
 typedef enum _nm_packed {
     NM_L3CFG_CONFIG_FLAGS_NONE               = 0,
     NM_L3CFG_CONFIG_FLAGS_ONLY_FOR_ACD       = (1LL << 0),
     NM_L3CFG_CONFIG_FLAGS_ASSUME_CONFIG_ONCE = (1LL << 1),
-    NM_L3CFG_CONFIG_FLAGS_FORCE_ONCE         = (1LL << 2),
 } NML3CfgConfigFlags;
 
 typedef enum _nm_packed {
@@ -95,6 +92,7 @@ typedef struct {
     NML3AcdAddrState            state;
     NML3Cfg                    *l3cfg;
     const NML3AcdAddrTrackInfo *track_infos;
+    NMEtherAddr                 last_conflict_addr;
 } NML3AcdAddrInfo;
 
 static inline const NML3AcdAddrTrackInfo *
@@ -131,8 +129,6 @@ typedef enum {
      * and neither should you call into NML3Cfg again (reentrancy). */
     NM_L3_CONFIG_NOTIFY_TYPE_L3CD_CHANGED,
 
-    NM_L3_CONFIG_NOTIFY_TYPE_ROUTES_TEMPORARY_NOT_AVAILABLE_EXPIRED,
-
     NM_L3_CONFIG_NOTIFY_TYPE_ACD_EVENT,
 
     /* emitted before the merged l3cd is committed to platform.
@@ -195,19 +191,29 @@ typedef struct {
 } NML3ConfigNotifyData;
 
 struct _NML3CfgPrivate;
-struct _NMPRouteManager;
+struct _NMPGlobalTracker;
 
 struct _NML3Cfg {
     GObject parent;
     struct {
-        struct _NML3CfgPrivate  *p;
-        NMNetns                 *netns;
-        NMPlatform              *platform;
-        struct _NMPRouteManager *route_manager;
-        const NMPObject         *plobj;
-        const NMPObject         *plobj_next;
-        int                      ifindex;
+        struct _NML3CfgPrivate   *p;
+        NMNetns                  *netns;
+        NMPlatform               *platform;
+        struct _NMPGlobalTracker *global_tracker;
+        const NMPObject          *plobj;
+        const NMPObject          *plobj_next;
+        int                       ifindex;
     } priv;
+
+    /* NML3Cfg strongly cooperates with NMNetns. The latter is
+     * the one that creates and manages (also) the lifetime of the
+     * NML3Cfg instance. We track some per-l3cfg-data that is only
+     * relevant to NMNetns here. */
+    struct {
+        guint32 signal_pending_obj_type_flags;
+        CList   signal_pending_lst;
+        CList   ecmp_track_ifindex_lst_head;
+    } internal_netns;
 };
 
 typedef struct _NML3CfgClass NML3CfgClass;
@@ -363,15 +369,6 @@ typedef enum _nm_packed {
     /* Don't touch the interface. */
     NM_L3_CFG_COMMIT_TYPE_NONE,
 
-    /* ASSUME means to keep any pre-existing extra routes/addresses, while
-     * also not adding routes/addresses that are not present yet. This is to
-     * gracefully take over after restart, where the existing IP configuration
-     * should not change.
-     *
-     * The flag NM_L3CFG_CONFIG_FLAGS_ASSUME_CONFIG_ONCE can make certain addresses/
-     * routes commitable also during "assume". */
-    NM_L3_CFG_COMMIT_TYPE_ASSUME,
-
     /* UPDATE means to add new addresses/routes, while also removing addresses/routes
      * that are no longer present (but were previously configured by NetworkManager).
      * Routes/addresses that were removed externally won't be re-added, and routes/addresses
@@ -408,9 +405,9 @@ gboolean nm_l3cfg_check_ready(NML3Cfg               *self,
                               const NML3ConfigData  *l3cd,
                               int                    addr_family,
                               NML3CfgCheckReadyFlags flags,
-                              gboolean              *acd_used);
+                              GArray               **conflicts);
 
-gboolean nm_l3cfg_has_temp_not_available_obj(NML3Cfg *self, int addr_family);
+gboolean nm_l3cfg_has_failedobj_pending(NML3Cfg *self, int addr_family);
 
 /*****************************************************************************/
 
@@ -464,4 +461,9 @@ struct _NMIPConfig *nm_l3cfg_ipconfig_acquire(NML3Cfg *self, int addr_family);
 
 /*****************************************************************************/
 
+typedef struct _NML3CfgBlockHandle NML3CfgBlockHandle;
+
+NML3CfgBlockHandle *nm_l3cfg_block_obj_pruning(NML3Cfg *self, int addr_family);
+void                nm_l3cfg_unblock_obj_pruning(NML3CfgBlockHandle *handle);
+
 #endif /* __NM_L3CFG_H__ */
