@@ -69,6 +69,8 @@ NMManager *nm_manager_setup(void);
 NMManager *nm_manager_get(void);
 #define NM_MANAGER_GET (nm_manager_get())
 
+NMPolicy *nm_manager_get_policy(NMManager *self);
+
 gboolean nm_manager_start(NMManager *manager, GError **error);
 void     nm_manager_stop(NMManager *manager);
 NMState  nm_manager_get_state(NMManager *manager);
@@ -121,6 +123,10 @@ NMSettingsConnection **nm_manager_get_activatable_connections(NMManager *manager
                                                               gboolean   sort,
                                                               guint     *out_len);
 
+void nm_manager_deactivate_ac(NMManager *self, NMSettingsConnection *connection);
+
+void nm_manager_device_recheck_auto_activate_schedule(NMManager *self, NMDevice *device);
+
 void     nm_manager_write_device_state_all(NMManager *manager);
 gboolean nm_manager_write_device_state(NMManager *manager, NMDevice *device, int *out_ifindex);
 
@@ -207,6 +213,11 @@ struct _NMDnsManager *nm_manager_get_dns_manager(NMManager *self);
 
 /*****************************************************************************/
 
+void nm_manager_notify_delete_settings_connections(NMManager            *self,
+                                                   NMSettingsConnection *sett_conn);
+
+/*****************************************************************************/
+
 void nm_manager_device_auth_request(NMManager                     *self,
                                     NMDevice                      *device,
                                     GDBusMethodInvocation         *context,
@@ -219,4 +230,40 @@ void nm_manager_device_auth_request(NMManager                     *self,
 
 void nm_manager_unblock_failed_ovs_interfaces(NMManager *self);
 
+/*****************************************************************************/
+
+#define NM_AUTOCONNECT_RETRIES_FOREVER G_MAXUINT32
+
+guint32 nm_manager_devcon_autoconnect_retries_get(NMManager            *self,
+                                                  NMDevice             *device,
+                                                  NMSettingsConnection *sett_conn);
+
+void nm_manager_devcon_autoconnect_retries_set(NMManager            *self,
+                                               NMDevice             *device,
+                                               NMSettingsConnection *sett_conn,
+                                               guint32               retries);
+
+gboolean nm_manager_devcon_autoconnect_retries_reset(NMManager            *self,
+                                                     NMDevice             *device,
+                                                     NMSettingsConnection *sett_conn);
+
+gboolean nm_manager_devcon_autoconnect_reset_reconnect_all(NMManager            *self,
+                                                           NMDevice             *device,
+                                                           NMSettingsConnection *sett_conn,
+                                                           gboolean              only_no_secrets);
+
+gint32 nm_manager_devcon_autoconnect_retries_blocked_until(NMManager            *self,
+                                                           NMDevice             *device,
+                                                           NMSettingsConnection *sett_conn);
+
+gboolean nm_manager_devcon_autoconnect_is_blocked(NMManager            *self,
+                                                  NMDevice             *device,
+                                                  NMSettingsConnection *sett_conn);
+
+gboolean nm_manager_devcon_autoconnect_blocked_reason_set(NMManager            *self,
+                                                          NMDevice             *device,
+                                                          NMSettingsConnection *sett_conn,
+                                                          NMSettingsAutoconnectBlockedReason value,
+                                                          gboolean                           set);
+
 #endif /* __NETWORKMANAGER_MANAGER_H__ */
