@@ -13,7 +13,7 @@
 /*****************************************************************************/
 
 const char **
-nm_utils_bond_option_arp_ip_targets_split(const char *arp_ip_target)
+nm_utils_bond_option_ip_split(const char *arp_ip_target)
 {
     return nm_strsplit_set_full(arp_ip_target, ",", NM_STRSPLIT_SET_FLAGS_STRSTRIP);
 }
@@ -36,6 +36,7 @@ _nm_setting_bond_remove_options_arp_interval(NMSettingBond *s_bond)
 
     nm_setting_bond_remove_option(s_bond, NM_SETTING_BOND_OPTION_ARP_INTERVAL);
     nm_setting_bond_remove_option(s_bond, NM_SETTING_BOND_OPTION_ARP_IP_TARGET);
+    nm_setting_bond_remove_option(s_bond, NM_SETTING_BOND_OPTION_NS_IP6_TARGET);
 }
 
 /*****************************************************************************/
@@ -86,6 +87,20 @@ NM_UTILS_STRING_TABLE_LOOKUP_DEFINE(
     {"follow", NM_BOND_FAIL_OVER_MAC_FOLLOW},
     {"none", NM_BOND_FAIL_OVER_MAC_NONE}, );
 
+NM_UTILS_STRING_TABLE_LOOKUP_DEFINE(
+    _nm_setting_bond_lacp_active_from_string,
+    NMBondLacpActive,
+    {
+        G_STATIC_ASSERT_EXPR(_NM_BOND_LACP_ACTIVE_NUM <= 2);
+
+        if (name && name[0] < '0' + _NM_BOND_LACP_ACTIVE_NUM && name[0] >= '0' && name[1] == '\0') {
+            return name[0] - '0';
+        }
+    },
+    { return NM_BOND_LACP_ACTIVE_ON; },
+    {"off", NM_BOND_LACP_ACTIVE_OFF},
+    {"on", NM_BOND_LACP_ACTIVE_ON}, );
+
 NM_UTILS_STRING_TABLE_LOOKUP_DEFINE(
     _nm_setting_bond_lacp_rate_from_string,
     NMBondLacpRate,
@@ -446,6 +461,23 @@ nm_utils_validate_dhcp4_vendor_class_id(const char *vci, GError **error)
     return TRUE;
 }
 
+gboolean
+nm_utils_validate_dhcp_dscp(const char *dscp, GError **error)
+{
+    g_return_val_if_fail(!error || !(*error), FALSE);
+    g_return_val_if_fail(dscp, FALSE);
+
+    if (!NM_IN_STRSET(dscp, "CS0", "CS4", "CS6")) {
+        g_set_error_literal(error,
+                            NM_CONNECTION_ERROR,
+                            NM_CONNECTION_ERROR_INVALID_PROPERTY,
+                            _("invalid DSCP value; allowed values are: 'CS0', 'CS4', 'CS6'"));
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 gboolean
 nm_settings_connection_validate_permission_user(const char *item, gssize len)
 {
@@ -635,7 +667,7 @@ nm_utils_dnsname_normalize(int addr_family, const char *dns, char **out_free)
 
 /**
  * nm_setting_ovs_other_config_check_key:
- * @key: (allow-none): the key to check
+ * @key: (nullable): the key to check
  * @error: a #GError, %NULL to ignore.
  *
  * Checks whether @key is a valid key for OVS' other-config.
@@ -689,7 +721,7 @@ nm_setting_ovs_other_config_check_key(const char *key, GError **error)
 
 /**
  * nm_setting_ovs_other_config_check_val:
- * @val: (allow-none): the value to check
+ * @val: (nullable): the value to check
  * @error: a #GError, %NULL to ignore.
  *
  * Checks whether @val is a valid user data value. This means,
