@@ -32,11 +32,11 @@ NM_GOBJECT_PROPERTIES_DEFINE(NMSettingMatch,
  * Since: 1.14
  */
 struct _NMSettingMatch {
-    NMSetting parent;
-    GArray   *interface_name;
-    GArray   *kernel_command_line;
-    GArray   *driver;
-    GArray   *path;
+    NMSetting   parent;
+    NMValueStrv interface_name;
+    NMValueStrv kernel_command_line;
+    NMValueStrv driver;
+    NMValueStrv path;
 };
 
 struct _NMSettingMatchClass {
@@ -60,7 +60,7 @@ nm_setting_match_get_num_interface_names(NMSettingMatch *setting)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), 0);
 
-    return nm_g_array_len(setting->interface_name);
+    return nm_g_array_len(setting->interface_name.arr);
 }
 
 /**
@@ -68,6 +68,8 @@ nm_setting_match_get_num_interface_names(NMSettingMatch *setting)
  * @setting: the #NMSettingMatch
  * @idx: index number of the DNS search domain to return
  *
+ * Since 1.46, access at index "len" is allowed and returns NULL.
+ *
  * Returns: the interface name at index @idx
  *
  * Since: 1.14
@@ -77,10 +79,7 @@ nm_setting_match_get_interface_name(NMSettingMatch *setting, int idx)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    g_return_val_if_fail(setting->interface_name && idx >= 0 && idx < setting->interface_name->len,
-                         NULL);
-
-    return nm_strvarray_get_idx(setting->interface_name, idx);
+    return nm_strvarray_get_idxnull_or_greturn(setting->interface_name.arr, idx);
 }
 
 /**
@@ -98,7 +97,7 @@ nm_setting_match_add_interface_name(NMSettingMatch *setting, const char *interfa
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
     g_return_if_fail(interface_name);
 
-    nm_strvarray_add(nm_strvarray_ensure(&setting->interface_name), interface_name);
+    nm_strvarray_ensure_and_add(&setting->interface_name.arr, interface_name);
     _notify(setting, PROP_INTERFACE_NAME);
 }
 
@@ -116,9 +115,10 @@ nm_setting_match_remove_interface_name(NMSettingMatch *setting, int idx)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    g_return_if_fail(setting->interface_name && idx >= 0 && idx < setting->interface_name->len);
+    g_return_if_fail(setting->interface_name.arr && idx >= 0
+                     && idx < setting->interface_name.arr->len);
 
-    g_array_remove_index(setting->interface_name, idx);
+    nm_strvarray_remove_index(setting->interface_name.arr, idx);
     _notify(setting, PROP_INTERFACE_NAME);
 }
 
@@ -139,12 +139,11 @@ nm_setting_match_remove_interface_name_by_value(NMSettingMatch *setting, const c
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), FALSE);
     g_return_val_if_fail(interface_name, FALSE);
 
-    if (nm_strvarray_remove_first(setting->interface_name, interface_name)) {
-        _notify(setting, PROP_INTERFACE_NAME);
-        return TRUE;
-    }
+    if (!nm_strvarray_remove_first(setting->interface_name.arr, interface_name))
+        return FALSE;
 
-    return FALSE;
+    _notify(setting, PROP_INTERFACE_NAME);
+    return TRUE;
 }
 
 /**
@@ -160,16 +159,14 @@ nm_setting_match_clear_interface_names(NMSettingMatch *setting)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    if (nm_g_array_len(setting->interface_name) != 0) {
-        nm_clear_pointer(&setting->interface_name, g_array_unref);
+    if (nm_strvarray_clear(&setting->interface_name.arr))
         _notify(setting, PROP_INTERFACE_NAME);
-    }
 }
 
 /**
  * nm_setting_match_get_interface_names:
  * @setting: the #NMSettingMatch
- * @length: (out) (allow-none): the length of the returned interface names array.
+ * @length: (out) (optional): the length of the returned interface names array.
  *
  * Returns all the interface names.
  *
@@ -185,7 +182,7 @@ nm_setting_match_get_interface_names(NMSettingMatch *setting, guint *length)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    return nm_strvarray_get_strv(&setting->interface_name, length);
+    return nm_strvarray_get_strv_notnull(setting->interface_name.arr, length);
 }
 
 /*****************************************************************************/
@@ -203,7 +200,7 @@ nm_setting_match_get_num_kernel_command_lines(NMSettingMatch *setting)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), 0);
 
-    return nm_g_array_len(setting->kernel_command_line);
+    return nm_g_array_len(setting->kernel_command_line.arr);
 }
 
 /**
@@ -211,6 +208,8 @@ nm_setting_match_get_num_kernel_command_lines(NMSettingMatch *setting)
  * @setting: the #NMSettingMatch
  * @idx: index number of the kernel command line argument to return
  *
+ * Since 1.46, access at index "len" is allowed and returns NULL.
+ *
  * Returns: the kernel command line argument at index @idx
  *
  * Since: 1.26
@@ -220,10 +219,7 @@ nm_setting_match_get_kernel_command_line(NMSettingMatch *setting, guint idx)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    g_return_val_if_fail(setting->kernel_command_line && idx < setting->kernel_command_line->len,
-                         NULL);
-
-    return nm_strvarray_get_idx(setting->kernel_command_line, idx);
+    return nm_strvarray_get_idxnull_or_greturn(setting->kernel_command_line.arr, idx);
 }
 
 /**
@@ -241,7 +237,7 @@ nm_setting_match_add_kernel_command_line(NMSettingMatch *setting, const char *ke
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
     g_return_if_fail(kernel_command_line);
 
-    nm_strvarray_add(nm_strvarray_ensure(&setting->kernel_command_line), kernel_command_line);
+    nm_strvarray_ensure_and_add(&setting->kernel_command_line.arr, kernel_command_line);
     _notify(setting, PROP_KERNEL_COMMAND_LINE);
 }
 
@@ -259,9 +255,10 @@ nm_setting_match_remove_kernel_command_line(NMSettingMatch *setting, guint idx)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    g_return_if_fail(setting->kernel_command_line && idx < setting->kernel_command_line->len);
+    g_return_if_fail(setting->kernel_command_line.arr
+                     && idx < setting->kernel_command_line.arr->len);
 
-    g_array_remove_index(setting->kernel_command_line, idx);
+    nm_strvarray_remove_index(setting->kernel_command_line.arr, idx);
     _notify(setting, PROP_KERNEL_COMMAND_LINE);
 }
 
@@ -283,12 +280,11 @@ nm_setting_match_remove_kernel_command_line_by_value(NMSettingMatch *setting,
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), FALSE);
     g_return_val_if_fail(kernel_command_line, FALSE);
 
-    if (nm_strvarray_remove_first(setting->kernel_command_line, kernel_command_line)) {
-        _notify(setting, PROP_KERNEL_COMMAND_LINE);
-        return TRUE;
-    }
+    if (!nm_strvarray_remove_first(setting->kernel_command_line.arr, kernel_command_line))
+        return FALSE;
 
-    return FALSE;
+    _notify(setting, PROP_KERNEL_COMMAND_LINE);
+    return TRUE;
 }
 
 /**
@@ -304,16 +300,14 @@ nm_setting_match_clear_kernel_command_lines(NMSettingMatch *setting)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    if (nm_g_array_len(setting->kernel_command_line) != 0) {
-        nm_clear_pointer(&setting->kernel_command_line, g_array_unref);
+    if (nm_strvarray_clear(&setting->kernel_command_line.arr))
         _notify(setting, PROP_KERNEL_COMMAND_LINE);
-    }
 }
 
 /**
  * nm_setting_match_get_kernel_command_lines:
  * @setting: the #NMSettingMatch
- * @length: (out) (allow-none): the length of the returned interface names array.
+ * @length: (out) (optional): the length of the returned interface names array.
  *
  * Returns all the interface names.
  *
@@ -326,7 +320,7 @@ nm_setting_match_get_kernel_command_lines(NMSettingMatch *setting, guint *length
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    return nm_strvarray_get_strv(&setting->kernel_command_line, length);
+    return nm_strvarray_get_strv_notnull(setting->kernel_command_line.arr, length);
 }
 
 /*****************************************************************************/
@@ -344,7 +338,7 @@ nm_setting_match_get_num_drivers(NMSettingMatch *setting)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), 0);
 
-    return nm_g_array_len(setting->driver);
+    return nm_g_array_len(setting->driver.arr);
 }
 
 /**
@@ -352,6 +346,8 @@ nm_setting_match_get_num_drivers(NMSettingMatch *setting)
  * @setting: the #NMSettingMatch
  * @idx: index number of the DNS search domain to return
  *
+ * Since 1.46, access at index "len" is allowed and returns NULL.
+ *
  * Returns: the driver at index @idx
  *
  * Since: 1.26
@@ -361,9 +357,7 @@ nm_setting_match_get_driver(NMSettingMatch *setting, guint idx)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    g_return_val_if_fail(setting->driver && idx < setting->driver->len, NULL);
-
-    return nm_strvarray_get_idx(setting->driver, idx);
+    return nm_strvarray_get_idxnull_or_greturn(setting->driver.arr, idx);
 }
 
 /**
@@ -381,7 +375,7 @@ nm_setting_match_add_driver(NMSettingMatch *setting, const char *driver)
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
     g_return_if_fail(driver);
 
-    nm_strvarray_add(nm_strvarray_ensure(&setting->driver), driver);
+    nm_strvarray_ensure_and_add(&setting->driver.arr, driver);
     _notify(setting, PROP_DRIVER);
 }
 
@@ -399,9 +393,9 @@ nm_setting_match_remove_driver(NMSettingMatch *setting, guint idx)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    g_return_if_fail(setting->driver && idx < setting->driver->len);
+    g_return_if_fail(setting->driver.arr && idx < setting->driver.arr->len);
 
-    g_array_remove_index(setting->driver, idx);
+    nm_strvarray_remove_index(setting->driver.arr, idx);
     _notify(setting, PROP_DRIVER);
 }
 
@@ -422,12 +416,11 @@ nm_setting_match_remove_driver_by_value(NMSettingMatch *setting, const char *dri
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), FALSE);
     g_return_val_if_fail(driver, FALSE);
 
-    if (nm_strvarray_remove_first(setting->driver, driver)) {
-        _notify(setting, PROP_DRIVER);
-        return TRUE;
-    }
+    if (!nm_strvarray_remove_first(setting->driver.arr, driver))
+        return FALSE;
 
-    return FALSE;
+    _notify(setting, PROP_DRIVER);
+    return TRUE;
 }
 
 /**
@@ -443,16 +436,14 @@ nm_setting_match_clear_drivers(NMSettingMatch *setting)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    if (nm_g_array_len(setting->driver) != 0) {
-        nm_clear_pointer(&setting->driver, g_array_unref);
+    if (nm_strvarray_clear(&setting->driver.arr))
         _notify(setting, PROP_DRIVER);
-    }
 }
 
 /**
  * nm_setting_match_get_drivers:
  * @setting: the #NMSettingMatch
- * @length: (out) (allow-none): the length of the returned interface names array.
+ * @length: (out) (optional): the length of the returned interface names array.
  *
  * Returns all the drivers.
  *
@@ -465,7 +456,7 @@ nm_setting_match_get_drivers(NMSettingMatch *setting, guint *length)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    return nm_strvarray_get_strv(&setting->driver, length);
+    return nm_strvarray_get_strv_notnull(setting->driver.arr, length);
 }
 
 /*****************************************************************************/
@@ -483,7 +474,7 @@ nm_setting_match_get_num_paths(NMSettingMatch *setting)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), 0);
 
-    return nm_g_array_len(setting->path);
+    return nm_g_array_len(setting->path.arr);
 }
 
 /**
@@ -491,6 +482,8 @@ nm_setting_match_get_num_paths(NMSettingMatch *setting)
  * @setting: the #NMSettingMatch
  * @idx: index number of the path to return
  *
+ * Since 1.46, access at index "len" is allowed and returns NULL.
+ *
  * Returns: the path at index @idx
  *
  * Since: 1.26
@@ -500,9 +493,7 @@ nm_setting_match_get_path(NMSettingMatch *setting, guint idx)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    g_return_val_if_fail(setting->path && idx < setting->path->len, NULL);
-
-    return nm_strvarray_get_idx(setting->path, idx);
+    return nm_strvarray_get_idxnull_or_greturn(setting->path.arr, idx);
 }
 
 /**
@@ -520,7 +511,7 @@ nm_setting_match_add_path(NMSettingMatch *setting, const char *path)
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
     g_return_if_fail(path);
 
-    nm_strvarray_add(nm_strvarray_ensure(&setting->path), path);
+    nm_strvarray_ensure_and_add(&setting->path.arr, path);
     _notify(setting, PROP_PATH);
 }
 
@@ -538,9 +529,9 @@ nm_setting_match_remove_path(NMSettingMatch *setting, guint idx)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    g_return_if_fail(setting->path && idx < setting->path->len);
+    g_return_if_fail(setting->path.arr && idx < setting->path.arr->len);
 
-    g_array_remove_index(setting->path, idx);
+    nm_strvarray_remove_index(setting->path.arr, idx);
     _notify(setting, PROP_PATH);
 }
 
@@ -561,12 +552,11 @@ nm_setting_match_remove_path_by_value(NMSettingMatch *setting, const char *path)
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), FALSE);
     g_return_val_if_fail(path, FALSE);
 
-    if (nm_strvarray_remove_first(setting->path, path)) {
-        _notify(setting, PROP_PATH);
-        return TRUE;
-    }
+    if (!nm_strvarray_remove_first(setting->path.arr, path))
+        return FALSE;
 
-    return FALSE;
+    _notify(setting, PROP_PATH);
+    return TRUE;
 }
 
 /**
@@ -582,16 +572,14 @@ nm_setting_match_clear_paths(NMSettingMatch *setting)
 {
     g_return_if_fail(NM_IS_SETTING_MATCH(setting));
 
-    if (nm_g_array_len(setting->path) != 0) {
-        nm_clear_pointer(&setting->path, g_array_unref);
+    if (nm_strvarray_clear(&setting->path.arr))
         _notify(setting, PROP_PATH);
-    }
 }
 
 /**
  * nm_setting_match_get_paths:
  * @setting: the #NMSettingMatch
- * @length: (out) (allow-none): the length of the returned paths array.
+ * @length: (out) (optional): the length of the returned paths array.
  *
  * Returns all the paths.
  *
@@ -604,58 +592,7 @@ nm_setting_match_get_paths(NMSettingMatch *setting, guint *length)
 {
     g_return_val_if_fail(NM_IS_SETTING_MATCH(setting), NULL);
 
-    return nm_strvarray_get_strv(&setting->path, length);
-}
-
-/*****************************************************************************/
-
-static void
-get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
-{
-    NMSettingMatch *self = NM_SETTING_MATCH(object);
-
-    switch (prop_id) {
-    case PROP_INTERFACE_NAME:
-        g_value_take_boxed(value, nm_strvarray_get_strv_non_empty_dup(self->interface_name, NULL));
-        break;
-    case PROP_KERNEL_COMMAND_LINE:
-        g_value_take_boxed(value,
-                           nm_strvarray_get_strv_non_empty_dup(self->kernel_command_line, NULL));
-        break;
-    case PROP_DRIVER:
-        g_value_take_boxed(value, nm_strvarray_get_strv_non_empty_dup(self->driver, NULL));
-        break;
-    case PROP_PATH:
-        g_value_take_boxed(value, nm_strvarray_get_strv_non_empty_dup(self->path, NULL));
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-        break;
-    }
-}
-
-static void
-set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
-{
-    NMSettingMatch *self = NM_SETTING_MATCH(object);
-
-    switch (prop_id) {
-    case PROP_INTERFACE_NAME:
-        nm_strvarray_set_strv(&self->interface_name, g_value_get_boxed(value));
-        break;
-    case PROP_KERNEL_COMMAND_LINE:
-        nm_strvarray_set_strv(&self->kernel_command_line, g_value_get_boxed(value));
-        break;
-    case PROP_DRIVER:
-        nm_strvarray_set_strv(&self->driver, g_value_get_boxed(value));
-        break;
-    case PROP_PATH:
-        nm_strvarray_set_strv(&self->path, g_value_get_boxed(value));
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-        break;
-    }
+    return nm_strvarray_get_strv_notnull(setting->path.arr, length);
 }
 
 /*****************************************************************************/
@@ -698,9 +635,9 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
     NMSettingMatch *self = NM_SETTING_MATCH(setting);
     guint           i;
 
-    if (self->interface_name) {
-        for (i = 0; i < self->interface_name->len; i++) {
-            if (nm_str_is_empty(nm_strvarray_get_idx(self->interface_name, i))) {
+    if (self->interface_name.arr) {
+        for (i = 0; i < self->interface_name.arr->len; i++) {
+            if (nm_str_is_empty(nm_strvarray_get_idx(self->interface_name.arr, i))) {
                 g_set_error(error,
                             NM_CONNECTION_ERROR,
                             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -714,9 +651,9 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
         }
     }
 
-    if (self->kernel_command_line) {
-        for (i = 0; i < self->kernel_command_line->len; i++) {
-            if (nm_str_is_empty(nm_strvarray_get_idx(self->kernel_command_line, i))) {
+    if (self->kernel_command_line.arr) {
+        for (i = 0; i < self->kernel_command_line.arr->len; i++) {
+            if (nm_str_is_empty(nm_strvarray_get_idx(self->kernel_command_line.arr, i))) {
                 g_set_error(error,
                             NM_CONNECTION_ERROR,
                             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -730,9 +667,9 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
         }
     }
 
-    if (self->driver) {
-        for (i = 0; i < self->driver->len; i++) {
-            if (nm_str_is_empty(nm_strvarray_get_idx(self->driver, i))) {
+    if (self->driver.arr) {
+        for (i = 0; i < self->driver.arr->len; i++) {
+            if (nm_str_is_empty(nm_strvarray_get_idx(self->driver.arr, i))) {
                 g_set_error(error,
                             NM_CONNECTION_ERROR,
                             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -746,9 +683,9 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
         }
     }
 
-    if (self->path) {
-        for (i = 0; i < self->path->len; i++) {
-            if (nm_str_is_empty(nm_strvarray_get_idx(self->path, i))) {
+    if (self->path.arr) {
+        for (i = 0; i < self->path.arr->len; i++) {
+            if (nm_str_is_empty(nm_strvarray_get_idx(self->path.arr, i))) {
                 g_set_error(error,
                             NM_CONNECTION_ERROR,
                             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -765,28 +702,15 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
     return TRUE;
 }
 
-static void
-finalize(GObject *object)
-{
-    NMSettingMatch *self = NM_SETTING_MATCH(object);
-
-    nm_clear_pointer(&self->interface_name, g_array_unref);
-    nm_clear_pointer(&self->kernel_command_line, g_array_unref);
-    nm_clear_pointer(&self->driver, g_array_unref);
-    nm_clear_pointer(&self->path, g_array_unref);
-
-    G_OBJECT_CLASS(nm_setting_match_parent_class)->finalize(object);
-}
-
 static void
 nm_setting_match_class_init(NMSettingMatchClass *klass)
 {
-    GObjectClass   *object_class  = G_OBJECT_CLASS(klass);
-    NMSettingClass *setting_class = NM_SETTING_CLASS(klass);
+    GObjectClass   *object_class        = G_OBJECT_CLASS(klass);
+    NMSettingClass *setting_class       = NM_SETTING_CLASS(klass);
+    GArray         *properties_override = _nm_sett_info_property_override_create_array();
 
-    object_class->get_property = get_property;
-    object_class->set_property = set_property;
-    object_class->finalize     = finalize;
+    object_class->get_property = _nm_setting_property_get_property_direct;
+    object_class->set_property = _nm_setting_property_set_property_direct;
 
     setting_class->verify = verify;
 
@@ -810,12 +734,13 @@ nm_setting_match_class_init(NMSettingMatchClass *klass)
      *
      * Since: 1.14
      **/
-    obj_properties[PROP_INTERFACE_NAME] = g_param_spec_boxed(
-        NM_SETTING_MATCH_INTERFACE_NAME,
-        "",
-        "",
-        G_TYPE_STRV,
-        NM_SETTING_PARAM_FUZZY_IGNORE | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_strv(properties_override,
+                                            obj_properties,
+                                            NM_SETTING_MATCH_INTERFACE_NAME,
+                                            PROP_INTERFACE_NAME,
+                                            NM_SETTING_PARAM_FUZZY_IGNORE,
+                                            NMSettingMatch,
+                                            interface_name);
 
     /**
      * NMSettingMatch:kernel-command-line
@@ -834,12 +759,13 @@ nm_setting_match_class_init(NMSettingMatchClass *klass)
      *
      * Since: 1.26
      **/
-    obj_properties[PROP_KERNEL_COMMAND_LINE] = g_param_spec_boxed(
-        NM_SETTING_MATCH_KERNEL_COMMAND_LINE,
-        "",
-        "",
-        G_TYPE_STRV,
-        NM_SETTING_PARAM_FUZZY_IGNORE | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_strv(properties_override,
+                                            obj_properties,
+                                            NM_SETTING_MATCH_KERNEL_COMMAND_LINE,
+                                            PROP_KERNEL_COMMAND_LINE,
+                                            NM_SETTING_PARAM_FUZZY_IGNORE,
+                                            NMSettingMatch,
+                                            kernel_command_line);
 
     /**
      * NMSettingMatch:driver
@@ -852,12 +778,13 @@ nm_setting_match_class_init(NMSettingMatchClass *klass)
      *
      * Since: 1.26
      **/
-    obj_properties[PROP_DRIVER] = g_param_spec_boxed(
-        NM_SETTING_MATCH_DRIVER,
-        "",
-        "",
-        G_TYPE_STRV,
-        NM_SETTING_PARAM_FUZZY_IGNORE | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_strv(properties_override,
+                                            obj_properties,
+                                            NM_SETTING_MATCH_DRIVER,
+                                            PROP_DRIVER,
+                                            NM_SETTING_PARAM_FUZZY_IGNORE,
+                                            NMSettingMatch,
+                                            driver);
 
     /**
      * NMSettingMatch:path
@@ -892,14 +819,19 @@ nm_setting_match_class_init(NMSettingMatchClass *klass)
      * example: MATCH_PATH="pci-0000:01:00.0 pci-0000:0c:00.0"
      * ---end---
      */
-    obj_properties[PROP_PATH] = g_param_spec_boxed(NM_SETTING_MATCH_PATH,
-                                                   "",
-                                                   "",
-                                                   G_TYPE_STRV,
-                                                   NM_SETTING_PARAM_FUZZY_IGNORE | G_PARAM_READWRITE
-                                                       | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_strv(properties_override,
+                                            obj_properties,
+                                            NM_SETTING_MATCH_PATH,
+                                            PROP_PATH,
+                                            NM_SETTING_PARAM_FUZZY_IGNORE,
+                                            NMSettingMatch,
+                                            path);
 
     g_object_class_install_properties(object_class, _PROPERTY_ENUMS_LAST, obj_properties);
 
-    _nm_setting_class_commit(setting_class, NM_META_SETTING_TYPE_MATCH, NULL, NULL, 0);
+    _nm_setting_class_commit(setting_class,
+                             NM_META_SETTING_TYPE_MATCH,
+                             NULL,
+                             properties_override,
+                             0);
 }
