@@ -305,7 +305,7 @@ NM_UTILS_LOOKUP_STR_DEFINE(
     NM_UTILS_LOOKUP_ITEM(NM_DEVICE_STATE_UNKNOWN, N_("unknown")), );
 
 static NM_UTILS_LOOKUP_STR_DEFINE(
-    _device_state_to_string,
+    _device_state_externally_to_string,
     NMDeviceState,
     NM_UTILS_LOOKUP_DEFAULT(NULL),
     NM_UTILS_LOOKUP_ITEM(NM_DEVICE_STATE_PREPARE, N_("connecting (externally)")),
@@ -330,7 +330,7 @@ nmc_device_state_to_string_with_external(NMDevice *device)
 
     if ((ac = nm_device_get_active_connection(device))
         && NM_FLAGS_HAS(nm_active_connection_get_state_flags(ac), NM_ACTIVATION_STATE_FLAG_EXTERNAL)
-        && (s = _device_state_to_string(state)))
+        && (s = _device_state_externally_to_string(state)))
         return s;
 
     return nmc_device_state_to_string(state);
@@ -464,7 +464,9 @@ NM_UTILS_LOOKUP_STR_DEFINE(
     NM_UTILS_LOOKUP_ITEM(NM_DEVICE_STATE_REASON_SRIOV_CONFIGURATION_FAILED,
                          N_("Failed to configure SR-IOV parameters")),
     NM_UTILS_LOOKUP_ITEM(NM_DEVICE_STATE_REASON_PEER_NOT_FOUND,
-                         N_("The Wi-Fi P2P peer could not be found")), );
+                         N_("The Wi-Fi P2P peer could not be found")),
+    NM_UTILS_LOOKUP_ITEM(NM_DEVICE_STATE_REASON_DEVICE_HANDLER_FAILED,
+                         N_("The device handler dispatcher returned an error")), );
 
 NM_UTILS_LOOKUP_STR_DEFINE(
     nm_active_connection_state_reason_to_string,
@@ -712,6 +714,9 @@ nmc_print_qrcode(const char *str)
                 bool bottom = qrcodegen_getModule(qrcode, x, y + 1);
                 if (top) {
                     g_print(bottom ? " " : "\u2584");
+                } else if (y > size) {
+                    /* Print the last line (the bottom QR border) in light gray, no bg color */
+                    g_print("\033[0;37m\u2580");
                 } else {
                     g_print(bottom ? "\u2580" : "\u2588");
                 }
@@ -732,7 +737,7 @@ nmc_print_qrcode(const char *str)
  * Example of @passwd_file contents:
  *   wifi.psk:tajne heslo
  *   802-1x.password:krakonos
- *   802-11-wireless-security:leap-password:my leap password
+ *   802-11-wireless-security.leap-password:my leap password
  *
  * Returns: (transfer full): hash table with parsed passwords, or %NULL on an error
  */
