@@ -118,6 +118,7 @@ int n_dhcp4_client_probe_config_dup(NDhcp4ClientProbeConfig *config,
         dup->init_reboot = config->init_reboot;
         dup->requested_ip = config->requested_ip;
         dup->ms_start_delay = config->ms_start_delay;
+        dup->dscp = config->dscp;
 
         for (unsigned int i = 0; i < config->n_request_parameters; ++i)
                 dup->request_parameters[dup->n_request_parameters++] = config->request_parameters[i];
@@ -190,6 +191,19 @@ _c_public_ void n_dhcp4_client_probe_config_set_init_reboot(NDhcp4ClientProbeCon
         config->init_reboot = init_reboot;
 }
 
+/**
+ * n_dhcp4_client_probe_config_set_dscp() - set the IP DSCP value
+ * @config:                     configuration to operate on
+ * @dscp:                       value to set
+ *
+ * This sets the DSCP property of the configuration object, which specifies
+ * the DSCP value to set in the first six bits of the DS field in the IPv4
+ * header. If this function is not called, the DSCP will be set to CS0.
+ */
+_c_public_ void n_dhcp4_client_probe_config_set_dscp(NDhcp4ClientProbeConfig *config, uint8_t dscp) {
+        config->dscp = dscp & 0x3F;
+}
+
 /**
  * n_dhcp4_client_probe_config_set_requested_ip() - set requested-ip property
  * @config:                     configuration to operate on
@@ -995,14 +1009,13 @@ static int n_dhcp4_client_probe_transition_nak(NDhcp4ClientProbe *probe) {
         case N_DHCP4_CLIENT_PROBE_STATE_RENEWING:
         case N_DHCP4_CLIENT_PROBE_STATE_REBINDING:
 
-                /* XXX */
-
                 r = n_dhcp4_client_probe_raise(probe,
                                                NULL,
                                                N_DHCP4_CLIENT_EVENT_RETRACTED);
                 if (r)
                         return r;
 
+                probe->current_lease = n_dhcp4_client_lease_unref(probe->current_lease);
                 probe->state = N_DHCP4_CLIENT_PROBE_STATE_INIT;
                 probe->ns_deferred = n_dhcp4_gettime(CLOCK_BOOTTIME) + probe->ns_nak_restart_delay;
                 probe->ns_nak_restart_delay = C_CLAMP(probe->ns_nak_restart_delay * 2u,
@@ -1025,6 +1038,7 @@ static int n_dhcp4_client_probe_transition_nak(NDhcp4ClientProbe *probe) {
 
 int n_dhcp4_client_probe_transition_select(NDhcp4ClientProbe *probe, NDhcp4Incoming *offer, uint64_t ns_now) {
         _c_cleanup_(n_dhcp4_outgoing_freep) NDhcp4Outgoing *request = NULL;
+        NDhcp4ClientLease *l, *t_l;
         int r;
 
         switch (probe->state) {
@@ -1043,11 +1057,16 @@ int n_dhcp4_client_probe_transition_select(NDhcp4ClientProbe *probe, NDhcp4Incom
                 else
                         request = NULL; /* consumed */
 
-                /* XXX: ignore other offers */
-
                 probe->state = N_DHCP4_CLIENT_PROBE_STATE_REQUESTING;
 
-                break;
+                /*
+                 * Only one of the offered leases can be selected, so flush the list.
+                 * All offered lease, including this one are now dead.
+                 */
+                c_list_for_each_entry_safe(l, t_l, &probe->lease_list, probe_link)
+                        n_dhcp4_client_lease_unlink(l);
+
+                return 0;
         case N_DHCP4_CLIENT_PROBE_STATE_INIT:
         case N_DHCP4_CLIENT_PROBE_STATE_INIT_REBOOT:
         case N_DHCP4_CLIENT_PROBE_STATE_REBOOTING:
@@ -1058,11 +1077,9 @@ int n_dhcp4_client_probe_transition_select(NDhcp4ClientProbe *probe, NDhcp4Incom
         case N_DHCP4_CLIENT_PROBE_STATE_REBINDING:
         case N_DHCP4_CLIENT_PROBE_STATE_EXPIRED:
         default:
-                /* ignore */
-                break;
+                /* user called in invalid state. Return error. */
+                return -ENOTRECOVERABLE;
         }
-
-        return 0;
 }
 
 /**
@@ -1086,10 +1103,11 @@ int n_dhcp4_client_probe_transition_accept(NDhcp4ClientProbe *probe, NDhcp4Incom
                         return r;
 
                 probe->state = N_DHCP4_CLIENT_PROBE_STATE_BOUND;
-
+                probe->ns_decline_restart_delay = 0;
+                n_dhcp4_client_lease_unlink(probe->current_lease);
                 n_dhcp4_client_arm_timer(probe->client);
 
-                break;
+                return 0;
 
         case N_DHCP4_CLIENT_PROBE_STATE_INIT:
         case N_DHCP4_CLIENT_PROBE_STATE_INIT_REBOOT:
@@ -1101,11 +1119,9 @@ int n_dhcp4_client_probe_transition_accept(NDhcp4ClientProbe *probe, NDhcp4Incom
         case N_DHCP4_CLIENT_PROBE_STATE_REBINDING:
         case N_DHCP4_CLIENT_PROBE_STATE_EXPIRED:
         default:
-                /* ignore */
-                break;
+                /* user called in invalid state. Return error. */
+                return -ENOTRECOVERABLE;
         }
-
-        return 0;
 }
 
 /**
@@ -1127,9 +1143,22 @@ int n_dhcp4_client_probe_transition_decline(NDhcp4ClientProbe *probe, NDhcp4Inco
                 else
                         request = NULL; /* consumed */
 
-                /* XXX: what state to transition to? */
+                n_dhcp4_client_lease_unlink(probe->current_lease);
+                probe->current_lease = n_dhcp4_client_lease_unref(probe->current_lease);
 
-                break;
+                probe->state = N_DHCP4_CLIENT_PROBE_STATE_INIT;
+
+                /* RFC2131, 3.1, 5.) The client SHOULD wait a minimum of ten seconds before restarting
+                 * the configuration process to avoid excessive network traffic in case of looping.
+                 *
+                 * Let's go beyond that, and use an exponential backoff. */
+                probe->ns_decline_restart_delay = C_CLAMP(probe->ns_decline_restart_delay * 2u,
+                                                          UINT64_C(10)  * UINT64_C(1000000000),
+                                                          UINT64_C(300) * UINT64_C(1000000000));
+                probe->ns_deferred =  n_dhcp4_gettime(CLOCK_BOOTTIME) + probe->ns_decline_restart_delay;
+
+                n_dhcp4_client_arm_timer(probe->client);
+                return 0;
 
         case N_DHCP4_CLIENT_PROBE_STATE_INIT:
         case N_DHCP4_CLIENT_PROBE_STATE_INIT_REBOOT:
@@ -1141,11 +1170,9 @@ int n_dhcp4_client_probe_transition_decline(NDhcp4ClientProbe *probe, NDhcp4Inco
         case N_DHCP4_CLIENT_PROBE_STATE_REBINDING:
         case N_DHCP4_CLIENT_PROBE_STATE_EXPIRED:
         default:
-                /* ignore */
-                break;
+                /* user called in invalid state. Return error. */
+                return -ENOTRECOVERABLE;
         }
-
-        return 0;
 }
 
 /**
